@@ -4,7 +4,7 @@
 # Pod stripped from pm file by OODoc 2.00.
 package Apache::Solr::Result;
 use vars '$VERSION';
-$VERSION = '0.91';
+$VERSION = '0.92';
 
 
 use warnings;
@@ -123,7 +123,6 @@ sub nrSelected()
 sub selected($;$)
 {   my ($self, $rank, $client) = @_;
     my $dec    = $self->decoded;
-#warn Dumper $dec;
     my $result = $dec->{result}
         or panic "there are no results (yet)";
 
@@ -149,8 +148,9 @@ sub selected($;$)
 
 sub highlighted($)
 {   my ($self, $doc) = @_;
-    my $rank = $doc->rank;
-    my $hl   = $self->selectedPage($rank)->decoded->{highlighting}
+    my $rank   = $doc->rank;
+    my $pagenr = $self->selectedPageNr($rank);
+    my $hl     = $self->selectedPage($pagenr)->decoded->{highlighting}
         or error __x"there is no highlighting information in the result";
     Apache::Solr::Document->fromResult($hl->{$doc->uniqueId}, $rank);
 }
@@ -222,9 +222,8 @@ sub selectedPageLoad($;$)
     $page->_pageset($self->{ASR_pages});
 
     # put new page in shared table of pages
+    # no weaken here?  only the first request is captured in the main program.
     $self->{ASR_pages}[$pagenr] = $page;
-    weaken $self->{ASR_pages}[$pagenr];
-    $page;
 }
 
 
